@@ -1,8 +1,7 @@
-use getopts::{Fail, Options};
+use getopts::{Fail, Matches, Options};
 use std::{
     error::Error,
     fmt::{Debug, Display},
-    net::{SocketAddr, ToSocketAddrs},
     path::PathBuf,
     time::Duration,
 };
@@ -15,16 +14,16 @@ pub const OUT_CORPUS_FLAG: &str = "out-corpus";
 pub const NO_OUT_CORPUS_FLAG: &str = "no-out-corpus";
 pub const ARTIFACTS_FLAG: &str = "artifacts";
 pub const NO_ARTIFACTS_FLAG: &str = "no-artifacts";
-pub const CORPUS_SIZE_FLAG: &str = "corpus-size";
-pub const SOCK_ADDR_FLAG: &str = "socket-address";
+pub const STATS_FLAG: &str = "stats";
+pub const NO_STATS_FLAG: &str = "no-stats";
+pub const COMMAND_FLAG: &str = "command";
 
-pub const MAX_DURATION_FLAG: &str = "max-duration";
-pub const MAX_ITERATIONS_FLAG: &str = "max-iterations";
+pub const MAX_DURATION_FLAG: &str = "stop-after-duration";
+pub const MAX_ITERATIONS_FLAG: &str = "stop-after-iterations";
 pub const STOP_AFTER_FIRST_FAILURE_FLAG: &str = "stop-after-first-failure";
 
 pub const COMMAND_FUZZ: &str = "fuzz";
-pub const COMMAND_MINIFY_INPUT: &str = "tmin";
-pub const COMMAND_MINIFY_CORPUS: &str = "cmin";
+pub const COMMAND_MINIFY_INPUT: &str = "minify";
 pub const COMMAND_READ: &str = "read";
 
 #[derive(Clone)]
@@ -38,12 +37,12 @@ impl Default for DefaultArguments {
     }
 }
 
+/// The task that the fuzzer is asked to perform.
 #[derive(Debug, Clone)]
 pub enum FuzzerCommand {
     Fuzz,
     Read { input_file: PathBuf },
     MinifyInput { input_file: PathBuf },
-    MinifyCorpus { corpus_size: usize },
 }
 impl Default for FuzzerCommand {
     fn default() -> Self {
@@ -51,6 +50,7 @@ impl Default for FuzzerCommand {
     }
 }
 
+/// Various arguments given to the fuzzer, typically provided by the `cargo fuzzcheck` command line tool.
 #[derive(Debug, Clone)]
 pub struct Arguments {
     pub command: FuzzerCommand,
@@ -61,17 +61,25 @@ pub struct Arguments {
     pub corpus_in: Option<PathBuf>,
     pub corpus_out: Option<PathBuf>,
     pub artifacts_folder: Option<PathBuf>,
-    pub socket_address: Option<SocketAddr>,
+    pub stats_folder: Option<PathBuf>,
 }
 
+/// The command line argument parser used by the fuzz target and `cargo fuzzcheck`
 #[must_use]
 #[no_coverage]
 pub fn options_parser() -> Options {
     let mut options = Options::new();
 
     let defaults = DefaultArguments::default();
-
-    options.long_only(true);
+    options.optopt(
+        "",
+        COMMAND_FLAG,
+        &format!(
+            "the action to be performed (default: fuzz). --{} is required when using `{}`",
+            INPUT_FILE_FLAG, COMMAND_MINIFY_INPUT
+        ),
+        &format!("<{} | {}>", COMMAND_FUZZ, COMMAND_MINIFY_INPUT),
+    );
     options.optopt(
         "",
         MAX_DURATION_FLAG,
@@ -115,8 +123,13 @@ pub fn options_parser() -> Options {
         )
         .as_str(),
     );
-    options.optopt("", INPUT_FILE_FLAG, "file containing a JSON-encoded input", "PATH");
-    options.optopt("", CORPUS_SIZE_FLAG, "target size of the corpus", "N");
+    options.optopt("", STATS_FLAG, "folder where the statistics will be written", "PATH");
+    options.optflag(
+        "",
+        NO_STATS_FLAG,
+        format!("do not save statistics, overrides --{stats}", stats = STATS_FLAG).as_str(),
+    );
+    options.optopt("", INPUT_FILE_FLAG, "file containing a test case", "PATH");
     options.optopt(
         "",
         MAX_INPUT_CPLX_FLAG,
@@ -128,52 +141,60 @@ pub fn options_parser() -> Options {
         "N",
     );
 
-    options.optopt(
-        "",
-        SOCK_ADDR_FLAG,
-        "address of the TCP socket for communication between cargo-fuzzcheck and the fuzz target",
-        "127.0.0.1:0",
-    );
-
-    options.optflag("", "help", "print this help menu");
+    options.optflag("h", "help", "print this help menu");
 
     options
 }
 
 impl Arguments {
+    /// Create an `Arguments` from the parsed result of [`options_parser()`].
+    ///
+    /// ### Arguments
+    /// * `for_cargo_fuzzcheck` : true if this method is called within `cargo fuzzcheck`, false otherwise.
+    ///   This is because `cargo fuzzcheck` also needs a fuzz target as argument, while the fuzzed binary
+    ///   does not.
     #[no_coverage]
-    pub fn from_parser(options: &Options, args: &[&str]) -> Result<Self, ArgumentsError> {
-        let matches = options.parse(args)?;
-
-        if args.is_empty() {
-            return Err(ArgumentsError::Validation(
-                "A command must be given to cargo fuzzcheck.".to_string(),
-            ));
-        }
-
-        if matches.opt_present("help") {
+    pub fn from_matches(matches: &Matches, for_cargo_fuzzcheck: bool) -> Result<Self, ArgumentsError> {
+        if matches.opt_present("help") || matches.free.contains(&"help".to_owned()) {
             return Err(ArgumentsError::WantsHelp);
         }
 
-        if !matches!(
-            args[0],
-            COMMAND_FUZZ | COMMAND_READ | COMMAND_MINIFY_INPUT | COMMAND_MINIFY_CORPUS
-        ) {
+        if for_cargo_fuzzcheck {
+            if matches.free.is_empty() {
+                return Err(ArgumentsError::Validation(
+                    "A fuzz target must be given to cargo fuzzcheck.".to_string(),
+                ));
+            }
+        }
+
+        let command = matches.opt_str(COMMAND_FLAG).unwrap_or(COMMAND_FUZZ.to_owned());
+
+        let command = command.as_str();
+
+        if !matches!(command, COMMAND_FUZZ | COMMAND_READ | COMMAND_MINIFY_INPUT) {
             return Err(ArgumentsError::Validation(format!(
-                r#"The command {c} is not supported. It can either be ‘{fuzz}’, ‘{tmin}’, or ‘{cmin}’."#,
-                c = args[0],
+                r#"The command {c} is not supported. It can either be ‘{fuzz}’ or ‘{minify}’."#,
+                c = &matches.free[0],
                 fuzz = COMMAND_FUZZ,
-                tmin = COMMAND_MINIFY_INPUT,
-                cmin = COMMAND_MINIFY_CORPUS
+                minify = COMMAND_MINIFY_INPUT,
             )));
         }
 
         let max_input_cplx: Option<f64> = matches
             .opt_str(MAX_INPUT_CPLX_FLAG)
-            .and_then(|x| x.parse::<usize>().ok())
-            .map(|x| x as f64);
+            .and_then(
+                #[no_coverage]
+                |x| x.parse::<usize>().ok(),
+            )
+            .map(
+                #[no_coverage]
+                |x| x as f64,
+            );
 
-        let corpus_in: Option<PathBuf> = matches.opt_str(IN_CORPUS_FLAG).and_then(|x| x.parse::<PathBuf>().ok());
+        let corpus_in: Option<PathBuf> = matches.opt_str(IN_CORPUS_FLAG).and_then(
+            #[no_coverage]
+            |x| x.parse::<PathBuf>().ok(),
+        );
 
         let no_in_corpus = if matches.opt_present(NO_IN_CORPUS_FLAG) {
             Some(())
@@ -181,7 +202,10 @@ impl Arguments {
             None
         };
 
-        let corpus_out: Option<PathBuf> = matches.opt_str(OUT_CORPUS_FLAG).and_then(|x| x.parse::<PathBuf>().ok());
+        let corpus_out: Option<PathBuf> = matches.opt_str(OUT_CORPUS_FLAG).and_then(
+            #[no_coverage]
+            |x| x.parse::<PathBuf>().ok(),
+        );
 
         let no_out_corpus = if matches.opt_present(NO_OUT_CORPUS_FLAG) {
             Some(())
@@ -189,7 +213,10 @@ impl Arguments {
             None
         };
 
-        let artifacts_folder: Option<PathBuf> = matches.opt_str(ARTIFACTS_FLAG).and_then(|x| x.parse::<PathBuf>().ok());
+        let artifacts_folder: Option<PathBuf> = matches.opt_str(ARTIFACTS_FLAG).and_then(
+            #[no_coverage]
+            |x| x.parse::<PathBuf>().ok(),
+        );
 
         let no_artifacts = if matches.opt_present(NO_ARTIFACTS_FLAG) {
             Some(())
@@ -197,20 +224,22 @@ impl Arguments {
             None
         };
 
-        let socket_address = matches.opt_str(SOCK_ADDR_FLAG).and_then(|x| {
-            if let Ok(mut addrs) = x.to_socket_addrs() {
-                addrs.next()
-            } else {
-                None
-            }
-        });
+        let stats_folder: Option<PathBuf> = matches.opt_str(STATS_FLAG).and_then(|x| x.parse::<PathBuf>().ok());
 
-        let input_file: Option<PathBuf> = matches.opt_str(INPUT_FILE_FLAG).and_then(|x| x.parse::<PathBuf>().ok());
-        let corpus_size: Option<usize> = matches.opt_str(CORPUS_SIZE_FLAG).and_then(|x| x.parse::<usize>().ok());
+        let no_stats = if matches.opt_present(NO_STATS_FLAG) {
+            Some(())
+        } else {
+            None
+        };
+
+        let input_file: Option<PathBuf> = matches.opt_str(INPUT_FILE_FLAG).and_then(
+            #[no_coverage]
+            |x| x.parse::<PathBuf>().ok(),
+        );
 
         // verify all the right options are here
 
-        let command = match args[0] {
+        let command = match command {
             COMMAND_FUZZ => FuzzerCommand::Fuzz,
             COMMAND_READ => {
                 let input_file = input_file.expect(&format!(
@@ -226,32 +255,25 @@ impl Arguments {
                 ));
                 FuzzerCommand::MinifyInput { input_file }
             }
-            COMMAND_MINIFY_CORPUS => {
-                let corpus_size = corpus_size.expect(&format!(
-                    "A corpus size must be provided when minifying a corpus. Use --{}",
-                    CORPUS_SIZE_FLAG
-                ));
-                if corpus_in.is_none() {
-                    panic!(
-                        "An input corpus must be provided when minifying a corpus. Use --{}",
-                        IN_CORPUS_FLAG
-                    )
-                }
-                FuzzerCommand::MinifyCorpus { corpus_size }
-            }
             _ => unreachable!(),
         };
 
         let maximum_duration = {
             let seconds = matches
                 .opt_str(MAX_DURATION_FLAG)
-                .and_then(|x| x.parse::<u64>().ok())
+                .and_then(
+                    #[no_coverage]
+                    |x| x.parse::<u64>().ok(),
+                )
                 .unwrap_or(u64::MAX);
             Duration::new(seconds, 0)
         };
         let maximum_iterations = matches
             .opt_str(MAX_ITERATIONS_FLAG)
-            .and_then(|x| x.parse::<usize>().ok())
+            .and_then(
+                #[no_coverage]
+                |x| x.parse::<usize>().ok(),
+            )
             .unwrap_or(usize::MAX);
         let stop_after_first_failure = matches.opt_present(STOP_AFTER_FIRST_FAILURE_FLAG);
 
@@ -274,8 +296,7 @@ impl Arguments {
         } else {
             artifacts_folder.clone()
         };
-
-        let socket_address = socket_address;
+        let stats_folder: Option<PathBuf> = if no_stats.is_some() { None } else { stats_folder.clone() };
 
         Ok(Arguments {
             command,
@@ -286,16 +307,18 @@ impl Arguments {
             corpus_in,
             corpus_out,
             artifacts_folder,
-            socket_address,
+            stats_folder,
         })
     }
 }
 
+/// The “help” output of cargo-fuzzcheck
+#[no_coverage]
 pub fn help(parser: &Options) -> String {
     let mut help = format!(
         r##"
 USAGE:
-    cargo-fuzzcheck <FUZZ_TEST> <SUBCOMMAND> [OPTIONS]
+    cargo-fuzzcheck <FUZZ_TEST> [OPTIONS]
     => Execute the subcommand on the given fuzz test.
 
 FUZZ_TEST:
@@ -315,15 +338,11 @@ FUZZ_TEST:
 
 SUBCOMMANDS:
     {fuzz}    Run the fuzz test
-    {tmin}    Minify a crashing test input, requires --{input_file}
-    {cmin}    Minify a corpus of test inputs, requires --{in_corpus} and --{corpus_size}
+    {minify}    Minify a crashing test input, requires --{input_file}
 "##,
         fuzz = COMMAND_FUZZ,
-        tmin = COMMAND_MINIFY_INPUT,
+        minify = COMMAND_MINIFY_INPUT,
         input_file = INPUT_FILE_FLAG,
-        cmin = COMMAND_MINIFY_CORPUS,
-        in_corpus = IN_CORPUS_FLAG,
-        corpus_size = CORPUS_SIZE_FLAG
     );
     help += parser.usage("").as_str();
     help += format!(
@@ -339,24 +358,16 @@ cargo-fuzzcheck target2 fuzz --{max_cplx} 4000 --{out_corpus} fuzz_results/out/
     and write the output corpus (i.e. the folder of most interesting test cases) 
     to fuzz_results/out/.
 
-cargo-fuzzcheck target1 {tmin} --{input_file} "artifacts/crash.json"
+cargo-fuzzcheck target1 {minify} --{input_file} "artifacts/crash.json"
     Using “target1”, minify the test input defined in the file 
     "artifacts/crash.json". It will put minified inputs in the folder 
     artifacts/crash.minified/ and name them {{complexity}}-{{hash}}.json. 
     For example, artifacts/crash.minified/4213--8cd7777109b57b8c.json
     is a minified input of complexity 42.13.
-
-cargo-fuzzcheck target1 {cmin} --{in_corpus} "fuzz-corpus" --{out_corpus} "minimized_corpus" --{corpus_size} 25
-    Using “target1”, minify the corpus defined by the folder "fuzz-corpus",
-    which should contain JSON-encoded test inputs. The 25 most important
-    inputs from fuzz-corpus will be copied to minimized-corpus.
 "#,
         fuzz = COMMAND_FUZZ,
-        tmin = COMMAND_MINIFY_INPUT,
+        minify = COMMAND_MINIFY_INPUT,
         input_file = INPUT_FILE_FLAG,
-        cmin = COMMAND_MINIFY_CORPUS,
-        in_corpus = IN_CORPUS_FLAG,
-        corpus_size = CORPUS_SIZE_FLAG,
         max_cplx = MAX_INPUT_CPLX_FLAG,
         out_corpus = OUT_CORPUS_FLAG,
     )
@@ -364,23 +375,26 @@ cargo-fuzzcheck target1 {cmin} --{in_corpus} "fuzz-corpus" --{out_corpus} "minim
     help
 }
 
+#[derive(Clone)]
 pub enum ArgumentsError {
-    NoArgumentsGiven(Options),
+    NoArgumentsGiven(String),
     Parsing(Fail),
     Validation(String),
     WantsHelp,
 }
 
 impl Debug for ArgumentsError {
+    #[no_coverage]
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         <Self as Display>::fmt(&self, f)
     }
 }
 impl Display for ArgumentsError {
+    #[no_coverage]
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
-            ArgumentsError::NoArgumentsGiven(parser) => {
-                write!(f, "No arguments were given.\nHelp:\n{}", help(parser))
+            ArgumentsError::NoArgumentsGiven(help) => {
+                write!(f, "No arguments were given.\nHelp:\n{}", help)
             }
             ArgumentsError::Parsing(e) => {
                 write!(
@@ -409,6 +423,7 @@ To display the help, run:
 impl Error for ArgumentsError {}
 
 impl From<Fail> for ArgumentsError {
+    #[no_coverage]
     fn from(e: Fail) -> Self {
         Self::Parsing(e)
     }
