@@ -17,9 +17,7 @@ pub enum FuzzerEvent {
     CrashNoInput,
     Pulse,
     Done,
-    New,
-    Replace(usize),
-    Remove(usize),
+    Replace(usize, usize),
     DidReadCorpus,
     CaughtSignal(i32),
     TestFailure,
